@@ -1,29 +1,28 @@
 <?php
 /**
- * The IMP_Folder:: class provides a set of methods for dealing with
- * folders, accounting for subscription, errors, etc.
+ * The IMP_Folder:: class provides a set of methods for dealing with folders,
+ * accounting for subscription, errors, etc.
  *
- * $Horde: imp/lib/Folder.php,v 1.130.10.2 2005/01/31 17:34:11 jan Exp $
+ * $Horde: imp/lib/Folder.php,v 1.130.10.12 2005/09/14 15:54:14 jan Exp $
  *
  * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 2000-2005 Jon Parise <jon@csh.rit.edu>
  *
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
+ * See the enclosed file COPYING for license information (GPL). If you did not
+ * receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@csh.rit.edu>
- * @version $Revision: 1.130.10.2 $
  * @since   IMP 2.3
  * @package IMP
  */
 class IMP_Folder {
 
     /**
-     * Keep around identical lists so that we don't hit the server more
-     * that once in the same page for the same thing.
+     * Keep around identical lists so that we don't hit the server more that
+     * once in the same page for the same thing.
      *
-     * @var array $_listCache
+     * @var array
      */
     var $_listCache = array();
 
@@ -31,28 +30,27 @@ class IMP_Folder {
      * Keep track of mailbox names that we have complained about to prevent
      * giving the user identical error messages.
      *
-     * @var array $_errorCache
+     * @var array
      */
     var $_errorCache = array();
 
     /**
      * Cached results from the exists() function.
      *
-     * @var array $existsResults
+     * @var array
      */
     var $_existsResults = array();
 
     /**
-     * Returns a reference to the global IMP_Folder object, only
-     * creating it if it doesn't already exist. This ensures that only
-     * one IMP_Folder instance is instantiated for any given session
+     * Returns a reference to the global IMP_Folder object, only creating it
+     * if it doesn't already exist. This ensures that only one IMP_Folder
+     * instance is instantiated for any given session.
      *
-     * This method must be invoked as:
+     * This method must be invoked as:<code>
      *   $imp_folder = &IMP_Folder::singleton();
+     * </code>
      *
-     * @access public
-     *
-     * @return object IMP_Folder  The IMP_Folder instance.
+     * @return IMP_Folder  The IMP_Folder instance.
      */
     function &singleton()
     {
@@ -66,34 +64,11 @@ class IMP_Folder {
     }
 
     /**
-     * List folders.
+     * Lists folders.
      *
-     * @access public
-     *
-     * @param resource $stream             An open connection to a mail server.
-     * @param string $server               The specification of the server to
-     *                                     connect to.
-     *                                     Ex: {mail.foo.com/pop3:110}.
-     * @param string $prefix               What to restrict the folder list
-     *                                     to. For instance, if the IMAP
-     *                                     server makes your entire home
-     *                                     directory available, but you only
-     *                                     want the folders in ~/mail/, then
-     *                                     this should be * 'mail/'.
-     * @param optional boolean $sub        Should we list only subscribed
-     *                                     folders?
-     * @param optional array $filter       An list of mailboxes that should be
-     *                                     left out of the list.
-     * @param optional array $hierarchies  An list of additional hierarchies
-     *                                     that are either outside prefix or
-     *                                     not advertised by default (#shared/
-     *                                     and similar with UW) to include in
-     *                                     the list.
-     * @param optional boolean $dotfiles   If false, do not include folders
-     *                                     beginning with '.'
-     * @param optional string $namespace   Namespace prefix prepended on folder
-     *                                     names (eg. "INBOX." in cyrus). We
-     *                                     remove this for cosmetic purposes.
+     * @param boolean $sub   Should we list only subscribed folders?
+     * @param array $filter  An list of mailboxes that should be left out of
+     *                       the list.
      *
      * @return array  An array of folders, where each array alement is an
      *                associative array containing three values: 'val', with
@@ -103,13 +78,16 @@ class IMP_Folder {
      *                'abbrev', containing a shortened (26 characters max)
      *                label for display in situations where space is short.
      */
-    function flist($stream, $server, $prefix, $sub = false, $filter = array(),
-                   $hierarchies = array(), $dotfiles = true, $namespace = '')
+    function flist($sub = false, $filter = array())
     {
         global $conf, $imp, $notification;
 
         $list = array();
 
+        $prefix = $imp['folders'];
+        $hierarchies = $imp['hierarchies'];
+        $namespace = $imp['namespace'];
+
         /* Make sure we're requesting all folders. */
         $folders = $prefix . '*';
 
@@ -117,11 +95,11 @@ class IMP_Folder {
         $listcmd = ($sub) ? 'imap_lsub' : 'imap_list';
 
         /* Compute values that will uniquely identify this list. */
-        $signature = implode('|', array($listcmd, $folders));
-        $full_signature = serialize(array($listcmd, $prefix, $filter, $hierarchies, $dotfiles, $namespace));
+        $signature = md5(serialize(array($listcmd, $folders)));
+        $full_signature = md5(serialize(array($listcmd, $prefix, $filter, $hierarchies, $namespace)));
 
-        /* Either get the list from the cache, or go to the IMAP
-           server to obtain it. */
+        /* Either get the list from the cache, or go to the IMAP server to
+           obtain it. */
         if ($conf['server']['cache_folders']) {
             require_once 'Horde/SessionObjects.php';
             $sessionOb = &Horde_SessionObjects::singleton();
@@ -140,13 +118,14 @@ class IMP_Folder {
         if (isset($this->_listCache[$signature])) {
             $mailboxes = $this->_listCache[$signature];
         } else {
-            if (!($mailboxes = $listcmd($stream, $server, $folders))) {
+            $server = IMP::serverString();
+            if (!($mailboxes = $listcmd($imp['stream'], $server, $folders))) {
                 $mailboxes = array();
             }
 
             if (is_array($hierarchies)) {
                 foreach ($hierarchies as $val) {
-                    $hboxes = $listcmd($stream, $server, $val . '*');
+                    $hboxes = $listcmd($imp['stream'], $server, $val . '*');
                     if (is_array($hboxes)) {
                         $mailboxes = array_merge($mailboxes, $hboxes);
                     }
@@ -201,7 +180,7 @@ class IMP_Folder {
                         $parentparts = $parts;
                         $pstack = array();
 
-                        while (($i = count($parentparts))) {
+                        while ($i = count($parentparts)) {
                             $parent = implode($imp['delimiter'], $parentparts);
                             if (!empty($parent) &&
                                 ($parent != 'INBOX') &&
@@ -235,7 +214,7 @@ class IMP_Folder {
                             $abbrev = $folded;
                         }
 
-                        if (($dotfiles || (strlen($label) > 0 && ($label[0] != '.' && !strstr($label, $imp['delimiter'] . '.'))))
+                        if (($imp['dotfiles'] || (strlen($label) > 0 && ($label[0] != '.' && !strstr($label, $imp['delimiter'] . '.'))))
                             && !in_array($mailbox, $filter)) {
                             $list[$mailbox] = array('val' => $mailbox, 'label' => $folded, 'abbrev' => $abbrev);
                         }
@@ -257,98 +236,24 @@ class IMP_Folder {
     }
 
     /**
-     * Return an array of folders. This is a wrapper around the
-     * flist() function which reduces the number of arguments needed if
-     * we can assume that IMP's full environment is present.
+     * Returns an array of folders. This is a wrapper around the flist()
+     * function which reduces the number of arguments needed if we can assume
+     * that IMP's full environment is present.
      *
-     * @access public
-     *
-     * @param optional array $filter  An array of mailboxes to ignore.
-     * @param optional boolean $sub   If set, will be used to determine if
-     *                                we should list only subscribed folders.
+     * @param array $filter  An array of mailboxes to ignore.
+     * @param boolean $sub   If set, will be used to determine if we should
+     *                       list only subscribed folders.
      *
      * @return array  The array of mailboxes returned by flist().
      */
     function flist_IMP($filter = array(), $sub = null)
     {
-        global $imp;
-
-        $sub = is_null($sub) ? $GLOBALS['prefs']->getValue('subscribe') : $sub;
-
-        return $this->flist($imp['stream'], IMP::serverString(), $imp['folders'], $sub, $filter, $imp['hierarchies'], $imp['dotfiles'], $imp['namespace']);
+        return $this->flist(is_null($sub) ? $GLOBALS['prefs']->getValue('subscribe') : $sub, $filter);
     }
 
     /**
-     * List all folders which the user is not subscribed to.
+     * Deletes one or more folders.
      *
-     * @access public
-     *
-     * @param resource $stream             An open connection to a mail server.
-     * @param string $server               The specification of the server to
-     *                                     connect to.
-     *                                     Ex: {mail.foo.com/pop3:110}.
-     * @param string $prefix               What to restrict the folder list to.
-     *                                     For instance, if the IMAP server
-     *                                     makes your entire home directory
-     *                                     available, but you only want the
-     *                                     folders in ~/mail/, then this should
-     *                                     be 'mail/'.
-     * @param optional array $filter       A list of mailboxes that should be
-     *                                     left out of the list.
-     * @param optional array $hierarchies  An list of additional hierarchies
-     *                                     that are either outside prefix or
-     *                                     not advertised by default (#shared/
-     *                                     and similar with UW) to include in
-     *                                     the list.
-     * @param optional boolean $dotfiles   If false, do not include folders
-     *                                     beginning with '.'
-     * @param optional string $namespace   Namespace prefix prepended on folder
-     *                                     names (eg. "INBOX." in cyrus). We
-     *                                     remove this for cosmetic purposes.
-     *
-     * @return array  An array of folders. (Format: see flist())
-     */
-    function flistUnsubscribed($stream, $server, $prefix, $filter = array(),
-                               $hierarchies = array(), $dotfiles = true,
-                               $namespace = '')
-    {
-        $sub = $this->flist($stream, $server, $prefix, true, $filter, $hierarchies, $dotfiles, $namespace);
-        $all = $this->flist($stream, $server, $prefix, false, $filter, $hierarchies, $dotfiles, $namespace);
-
-        $usub = array();
-        foreach (array_diff(array_keys($all), array_keys($sub)) as $key) {
-            $usub[$key] = $all[$key];
-        }
-
-        return $usub;
-    }
-
-    /**
-     * Return an array of unsubscribed folders. This is a wrapper around
-     * the flistUnsubscribed() function which reduces the number of
-     * arguments needed if we can assume that IMP's full environment is
-     * present.
-     *
-     * @access public
-     *
-     * @param optional array $filter  An array of mailboxes to ignore.
-     *
-     * @return array  The array of mailboxes returned by
-     *                flistUnsubscribed().
-     */
-    function flistUnsubscribed_IMP($filter = array())
-    {
-        global $imp;
-
-        return $this->flistUnsubscribed($imp['stream'], IMP::serverString(), $imp['folders'], $filter, $imp['hierarchies'], $imp['dotfiles'], $imp['namespace']);
-    }
-
-    /**
-     * Delete one or more folders.
-     *
-     * @access public
-     *
-     * @param resource $stream     A valid c-client stream.
      * @param array $folder_array  An array of full utf encoded folder names
      *                             to be deleted.
      * @param boolean $subscribe   A boolean describing whether or not to use
@@ -356,7 +261,7 @@ class IMP_Folder {
      *
      * @return boolean  Whether or not the folders were successfully deleted.
      */
-    function delete($stream, $folder_array, $subscribe)
+    function delete($folder_array, $subscribe)
     {
         global $conf, $imp, $notification;
 
@@ -365,18 +270,18 @@ class IMP_Folder {
         $deleted = array();
 
         if ($subscribe) {
-            $subscribed_folders = imap_listsubscribed($stream, $server, '*');
+            $subscribed_folders = imap_listsubscribed($imp['stream'], $server, '*');
         }
 
         foreach ($folder_array as $folder) {
-            if (!imap_deletemailbox($stream, $server . $folder)) {
+            if (!imap_deletemailbox($imp['stream'], $server . $folder)) {
                 $notification->push(sprintf(_("The folder \"%s\" was not deleted. This is what the server said"), IMP::displayFolder($folder)) .
                                     ': ' . imap_last_error(), 'horde.error');
                 $return_value = false;
             } else {
                 if ($subscribe &&
                     in_array($server . $folder, $subscribed_folders) &&
-                    !imap_unsubscribe($stream, $server . $folder)) {
+                    !imap_unsubscribe($imp['stream'], $server . $folder)) {
                     $notification->push(sprintf(_("The folder \"%s\" was deleted but you were not unsubscribed from it."), IMP::displayFolder($folder)), 'horde.warning');
                     $return_value = false;
                 } else {
@@ -405,30 +310,27 @@ class IMP_Folder {
     }
 
     /**
-     * Create a new IMAP folder if it does not already exist, and
-     * subcribe to it as well if requested.
+     * Create a new IMAP folder if it does not already exist, and subcribe to
+     * it as well if requested.
      *
-     * @access public
-     *
-     * @param resource $stream    A valid c-client stream.
      * @param string $folder      The full utf encoded folder to be created.
      * @param boolean $subscribe  A boolean describing whether or not to use
      *                            folder subscriptions.
      *
      * @return boolean  Whether or not the folder was successfully created.
      */
-    function create($stream, $folder, $subscribe)
+    function create($folder, $subscribe)
     {
         global $conf, $imp, $notification;
 
         /* Make sure we are not trying to create a duplicate folder */
-        if ($this->exists($stream, $folder)) {
+        if ($this->exists($imp['stream'], $folder)) {
             $notification->push(sprintf(_("The folder \"%s\" already exists"), IMP::displayFolder($folder)), 'horde.warning');
             return false;
         }
 
         /* Attempt to create the mailbox */
-        if (!imap_createmailbox($stream, IMP::serverString($folder))) {
+        if (!imap_createmailbox($imp['stream'], IMP::serverString($folder))) {
             $notification->push(sprintf(_("The folder \"%s\" was not created. This is what the server said"), IMP::displayFolder($folder)) .
                             ': ' . imap_last_error(), 'horde.error');
             return false;
@@ -441,7 +343,7 @@ class IMP_Folder {
 
         /* If the user uses SUBSCRIBE, then add to the subscribe list */
         if ($subscribe &&
-            !imap_subscribe($stream, IMP::serverString($folder))) {
+            !imap_subscribe($imp['stream'], IMP::serverString($folder))) {
             $notification->push(sprintf(_("The folder \"%s\" was created but you were not subscribed to it."), IMP::displayFolder($folder)), 'horde.warning');
         } else {
             /* The folder creation has been successful */
@@ -459,20 +361,18 @@ class IMP_Folder {
     }
 
     /**
-     * Find out if a specific folder exists or not.
+     * Finds out if a specific folder exists or not.
      *
-     * @param resource $stream           A valid c-client stream.
-     * @param string $folder             The full utf encoded folder name to
-     *                                   be checked.
-     * @param optional boolean $recheck  Should we recheck the folder?
+     * @param string $folder    The full utf encoded folder name to be checked.
+     * @param boolean $recheck  Should we recheck the folder?
      *
      * @return boolean  Whether or not the folder exists.
      */
-    function exists($stream, $folder, $recheck = false)
+    function exists($folder, $recheck = false)
     {
         if ($recheck || !isset($this->_existsResults[$folder])) {
             $path = IMP::serverString($folder);
-            $res = imap_list($stream, '', $path);
+            $res = imap_list($GLOBALS['imp']['stream'], '', $path);
             $this->_existsResults[$folder] = ($res ? in_array($path, $res) : false);
         }
 
@@ -480,17 +380,15 @@ class IMP_Folder {
     }
 
     /**
-     * Rename an IMAP folder. The subscription status remains the same.
-     * All subfolders will also be renamed.
+     * Renames an IMAP folder. The subscription status remains the same.  All
+     * subfolders will also be renamed.
      *
-     * @param resource $stream  A valid c-client stream.
-     * @param string $old       The old utf encoded folder name.
-     * @param string $new       The new utf encoded folder name.
+     * @param string $old  The old utf encoded folder name.
+     * @param string $new  The new utf encoded folder name.
      *
-     * @return boolean  Whether or not all folder(s) were successfully
-     *                  renamed.
+     * @return boolean  Whether or not all folder(s) were successfully renamed.
      */
-    function rename($stream, $old, $new)
+    function rename($old, $new)
     {
         global $conf, $imp, $notification;
 
@@ -500,13 +398,13 @@ class IMP_Folder {
 
         /* Get list of any folders that are underneath this one. */
         $all_folders = array($server . $old);
-        $folder_list = imap_list($stream, $server, $old . $imp['delimiter'] . '*');
+        $folder_list = imap_list($imp['stream'], $server, $old . $imp['delimiter'] . '*');
         if (is_array($folder_list)) {
             $all_folders = array_merge($folder_list, $all_folders);
         }
 
         /* Get subscribed status. */
-        $subscribed_folders = @imap_lsub($stream, $server, $old . '*');
+        $subscribed_folders = @imap_lsub($imp['stream'], $server, $old . '*');
         if (!$subscribed_folders) {
             $subscribed_folders = array();
         }
@@ -526,12 +424,12 @@ class IMP_Folder {
             /* Unsubscribe from current folder. */
             if (in_array($folder_old, $subscribed_folders)) {
                 $subscribe = true;
-                imap_unsubscribe($stream, $folder_old);
+                imap_unsubscribe($imp['stream'], $folder_old);
             }
 
-            if (imap_renamemailbox($stream, $folder_old, $folder_new)) {
+            if (imap_renamemailbox($imp['stream'], $folder_old, $folder_new)) {
                 if ($subscribe) {
-                    imap_subscribe($stream, $folder_new);
+                    imap_subscribe($imp['stream'], $folder_new);
                 }
 
                 $deleted[] = $name_old;
@@ -563,16 +461,15 @@ class IMP_Folder {
     }
 
     /**
-     * Subscribe to one or more IMAP folders.
+     * Subscribes to one or more IMAP folders.
      *
-     * @param resource $stream     A valid c-client stream.
      * @param array $folder_array  An array of full utf encoded folder names
      *                             to be subscribed.
      *
      * @return boolean  Whether or not the folders were successfully
      *                  subscribed to.
      */
-    function subscribe($stream, $folder_array)
+    function subscribe($folder_array)
     {
         global $conf, $imp, $notification;
 
@@ -586,7 +483,7 @@ class IMP_Folder {
 
         foreach ($folder_array as $folder) {
             if ($folder != ' ') {
-                if (!imap_subscribe($stream, IMP::serverString($folder))) {
+                if (!imap_subscribe($imp['stream'], IMP::serverString($folder))) {
                     $notification->push(sprintf(_("You were not subscribed to \"%s\". Here is what the server said"), IMP::displayFolder($folder)) . ': ' . imap_last_error(), 'horde.error');
                     $return_value = false;
                 } else {
@@ -614,16 +511,15 @@ class IMP_Folder {
     }
 
     /**
-     * Unsubscribe from one or more IMAP folders.
+     * Unsubscribes from one or more IMAP folders.
      *
-     * @param resource $stream     A valid c-client stream.
      * @param array $folder_array  An array of full utf encoded folder names
      *                             to be unsubscribed.
      *
      * @return boolean  Whether or not the folders were successfully
      *                  unsubscribed from.
      */
-    function unsubscribe($stream, $folder_array)
+    function unsubscribe($folder_array)
     {
         global $conf, $imp, $notification;
 
@@ -637,7 +533,9 @@ class IMP_Folder {
 
         foreach ($folder_array as $folder) {
             if ($folder != ' ') {
-                if (!imap_unsubscribe($stream, IMP::serverString($folder))) {
+                if (strcasecmp($folder, 'INBOX') == 0) {
+                    $notification->push(sprintf(_("You can not unsubscribe from \"%s\"."), IMP::displayFolder($folder)), 'horde.error');
+                } elseif (!imap_unsubscribe($imp['stream'], IMP::serverString($folder))) {
                     $notification->push(sprintf(_("You were not unsubscribed from \"%s\". Here is what the server said"), IMP::displayFolder($folder)) . ': ' . imap_last_error(), 'horde.error');
                     $return_value = false;
                 } else {
@@ -652,7 +550,7 @@ class IMP_Folder {
             require_once IMP_BASE . '/lib/IMAP/Tree.php';
             $imptree = &IMP_Tree::singleton();
             if ($imptree) {
-                $imptree->unsubscribe($folder);
+                $imptree->unsubscribe($unsubscribed);
             }
 
             /* Reset the folder cache. */
@@ -665,21 +563,20 @@ class IMP_Folder {
     }
 
     /**
-     * Generate a string that can be saved out to an mbox format
-     * mailbox file for a folder or set of folders, optionally
-     * including all subfolders of the selected folders as well. All
-     * folders will be put into the same string.
+     * Generates a string that can be saved out to an mbox format mailbox file
+     * for a folder or set of folders, optionally including all subfolders of
+     * the selected folders as well. All folders will be put into the same
+     * string.
      *
      * @author Didi Rieder <adrieder@sbox.tugraz.at>
      *
-     * @param resource $stream             A valid c-client stream.
-     * @param array $folder_list           A list of full utf encoded folder
-     *                                     names to generate an mbox file for.
-     * @param boolean optional $recursive  Include subfolders?
+     * @param array $folder_list  A list of full utf encoded folder names to
+     *                            generate an mbox file for.
+     * @param boolean $recursive  Include subfolders?
      *
      * @return string  An mbox format mailbox file.
      */
-    function generateMbox($stream, $folder_list, $recursive = false)
+    function &generateMbox($folder_list, $recursive = false)
     {
         $body = '';
 
@@ -688,9 +585,9 @@ class IMP_Folder {
             $imp_imap = &IMP_IMAP::singleton();
             foreach ($folder_list as $folder) {
                 $imp_imap->changeMbox($folder, OP_READONLY);
-                $count = imap_num_msg($stream);
+                $count = imap_num_msg($GLOBALS['imp']['stream']);
                 for ($i = 1; $i <= $count; $i++) {
-                    $h = imap_header($stream, $i);
+                    $h = imap_header($GLOBALS['imp']['stream'], $i);
                     $from = '<>';
                     if (isset($h->from[0])) {
                         if (isset($h->from[0]->mailbox) && isset($h->from[0]->host)) {
@@ -702,8 +599,8 @@ class IMP_Folder {
                        require a space in front of single digit days. */
                     $date = sprintf('%s %2s %s', date('D M', $h->udate), date('j', $h->udate), date('H:i:s Y', $h->udate));
                     $body .= 'From ' . $from . ' ' . $date . "\n";
-                    $body .= imap_fetchheader($stream, $i, FT_PREFETCHTEXT);
-                    $body .= imap_body($stream, $i) . "\n";
+                    $body .= imap_fetchheader($GLOBALS['imp']['stream'], $i, FT_PREFETCHTEXT);
+                    $body .= imap_body($GLOBALS['imp']['stream'], $i) . "\n";
                 }
             }
         }
@@ -713,26 +610,23 @@ class IMP_Folder {
     }
 
     /**
-     * Import messages into a given folder from a mbox format mailbox file.
+     * Imports messages into a given folder from a mbox format mailbox file.
      *
-     * @access public
-     *
-     * @param resource $stream  A valid c-client stream.
-     * @param string $folder    The folder to put the messages into.
-     * @param string $mbox      String containing the mbox filename.
+     * @param string $folder  The folder to put the messages into.
+     * @param string $mbox    String containing the mbox filename.
      *
      * @return mixed  False (boolean) on fail or the number of messages
      *                imported (integer) on success.
      */
-    function importMbox($stream, $folder, $mbox)
+    function importMbox($folder, $mbox)
     {
         $target = IMP::ServerString() . $folder;
 
         $message = '';
         $msgcount = 0;
 
-        $fd = fopen ($mbox, "r");
-        while (!feof ($fd)) {
+        $fd = fopen($mbox, "r");
+        while (!feof($fd)) {
             $line = fgets($fd, 4096);
 
             if (preg_match('/From .+@.+/A', $line)) {
@@ -741,7 +635,7 @@ class IMP_Folder {
                     $message = preg_replace("|([^\r])\n|", "\\1\r\n", $message);
                     $message = str_replace("\n\n", "\n\r\n", $message);
 
-                    if (imap_append($stream, $target, $message)) {
+                    if (imap_append($GLOBALS['imp']['stream'], $target, $message)) {
                         $msgcount++;
                     }
                 }
@@ -757,7 +651,7 @@ class IMP_Folder {
             $message = preg_replace("|([^\r])\n|", "\\1\r\n", $message);
             $message = str_replace("\n\n", "\n\r\n", $message);
 
-            if (imap_append($stream, $target, $message)) {
+            if (imap_append($GLOBALS['imp']['stream'], $target, $message)) {
                 $msgcount++;
             }
         }
