@@ -10,7 +10,7 @@ define('IMP_MESSAGE_COPY', 2);
  * handled in here so that code need not be repeated between mailbox, message,
  * and other pages.
  *
- * $Horde: imp/lib/Message.php,v 1.164.8.1 2005/01/03 12:25:33 jan Exp $
+ * $Horde: imp/lib/Message.php,v 1.164.8.8 2005/08/13 17:15:55 slusarz Exp $
  *
  * Copyright 2000-2001 Chris Hyde <chris@jeks.net>
  * Copyright 2000-2005 Chuck Hagenbuch <chuck@horde.org>
@@ -22,7 +22,6 @@ define('IMP_MESSAGE_COPY', 2);
  * @author  Chris Hyde <chris@jeks.net>
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.164.8.1 $
  * @since   IMP 2.3
  * @package IMP
  */
@@ -31,21 +30,19 @@ class IMP_Message {
     /**
      * The IMP_IMAP object for the current server.
      *
-     * @var object IMP_IMAP $_impImap
+     * @var IMP_IMAP
      */
     var $_impImap;
 
     /**
      * Using POP to access mailboxes?
      *
-     * @var boolean $_usepop
+     * @var boolean
      */
     var $_usepop = false;
 
     /**
      * Constructor.
-     *
-     * @access public
      */
     function IMP_Message()
     {
@@ -62,26 +59,14 @@ class IMP_Message {
     /**
      * Indices format:
      * ===============
-     * For any function below that requires an $indices parameter, the
-     * following inputs are allowed:
-     * 1. An array of messages indices in the following format:
-     *    msg_id IMP_IDX_SEP msg_folder
-     *    msg_id = Message index of the message
-     *    IMP_IDX_SEP = IMP constant used to separate index/folder
-     *    msg_folder = The full folder name containing the message index
-     * 2. An array with the full folder name as keys and an array of messsage
-     *    indices as the values.
-     * 3. An IMP_Mailbox object, which will use the current index/folder
-     *    as determined by the object. If an IMP_Mailbox object is used, it
-     *    will be updated after the action is performed.
+     * For any function below that requires an $indices parameter, see
+     * IMP::parseIndicesList() for the list of allowable inputs.
      */
 
     /**
      * Copies or moves a list of messages to a new folder.
      * Handles use of the IMP_SEARCH_MBOX mailbox and the Trash folder.
      *
-     * @access public
-     *
      * @param string $targetMbox  The mailbox to move/copy messages to.
      * @param integer $action     Either IMP_MESSAGE_MOVE or IMP_MESSAGE_COPY.
      * @param mixed &$indices     See above.
@@ -92,7 +77,7 @@ class IMP_Message {
     {
         global $imp, $notification, $prefs;
 
-        if (!($msgList = $this->_getMessageIndices($indices))) {
+        if (!($msgList = IMP::parseIndicesList($indices))) {
             return false;
         }
 
@@ -146,8 +131,6 @@ class IMP_Message {
      * Trash folder is being used.
      * Handles use of the IMP_SEARCH_MBOX mailbox and the Trash folder.
      *
-     * @access public
-     *
      * @param mixed &$indices   See above.
      * @param boolean $nuke     Override user preferences and nuke (i.e.
      *                          permanently delete) the messages instead?
@@ -160,7 +143,7 @@ class IMP_Message {
     {
         global $conf, $imp, $notification, $prefs;
 
-        if (!($msgList = $this->_getMessageIndices($indices))) {
+        if (!($msgList = IMP::parseIndicesList($indices))) {
             return false;
         }
 
@@ -180,8 +163,8 @@ class IMP_Message {
                 include_once IMP_BASE . '/lib/Folder.php';
                 $imp_folder = &IMP_Folder::singleton();
 
-                if (!$imp_folder->exists($imp['stream'], $trash)) {
-                    if (!$imp_folder->create($imp['stream'], $trash, $prefs->getValue('subscribe'))) {
+                if (!$imp_folder->exists($trash)) {
+                    if (!$imp_folder->create($trash, $prefs->getValue('subscribe'))) {
                         return false;
                     }
                 }
@@ -269,8 +252,6 @@ class IMP_Message {
      * Handles the IMP_SEARCH_MBOX mailbox.
      * This function works with IMAP only, not POP3.
      *
-     * @access public
-     *
      * @param mixed &$indices  See above.
      *
      * @return boolean  True if successful, false if not.
@@ -279,7 +260,7 @@ class IMP_Message {
     {
         global $imp, $notification;
 
-        if (!($msgList = $this->_getMessageIndices($indices))) {
+        if (!($msgList = IMP::parseIndicesList($indices))) {
             return false;
         }
 
@@ -310,8 +291,6 @@ class IMP_Message {
      * Copies or moves a list of messages to a tasklist.
      * Handles use of the IMP_SEARCH_MBOX mailbox and the Trash folder.
      *
-     * @access public
-     *
      * @param string    $tasklist   The tasklist in which the task will be
      *                              created.
      * @param integer   $action     Either IMP_MESSAGE_MOVE or IMP_MESSAGE_COPY.
@@ -323,7 +302,7 @@ class IMP_Message {
     {
         global $registry, $notification, $prefs;
 
-        if (!($msgList = $this->_getMessageIndices($indices))) {
+        if (!($msgList = IMP::parseIndicesList($indices))) {
             return false;
         }
 
@@ -412,21 +391,19 @@ class IMP_Message {
      * Strips a MIME Part out of a message.
      * Handles the IMP_SEARCH_MBOX mailbox.
      *
-     * @param object IMP_Mailbox &$imp_mailbox  The IMP_Mailbox object with
-     *                                          the current index set to the
-     *                                          message to be processed.
-     * @param string $partid                    The MIME ID of the part to
-     *                                          strip.
+     * @param IMP_Mailbox &$imp_mailbox  The IMP_Mailbox object with the
+     *                                   current index set to the message to be
+     *                                   processed.
+     * @param string $partid             The MIME ID of the part to strip.
      *
-     * @return boolean  Returns true if successful.
-     *                  Returns a PEAR_Error on error.
+     * @return boolean  Returns true on success, or PEAR_Error on error.
      */
     function stripPart(&$imp_mailbox, $partid)
     {
         global $imp;
 
         /* Return error if no index was provided. */
-        if (!($msgList = $this->_getMessageIndices($imp_mailbox))) {
+        if (!($msgList = IMP::parseIndicesList($imp_mailbox))) {
             return PEAR::raiseError('No index provided to IMP_Message::stripPart().');
         }
 
@@ -449,7 +426,7 @@ class IMP_Message {
         $message = $contents->getMIMEMessage();
         $oldPart = $message->getPart($partid);
         $newPart = new MIME_Part('text/plain');
-        $newPart->setContents('[' . _("Attachment stripped: Original attachment type") . ': "' . $oldPart->getType() . '", ' . _("name") . ': "' . $oldPart->getName(false, true) . '"]');
+        $newPart->setContents('[' . _("Attachment stripped: Original attachment type") . ': "' . $oldPart->getType() . '", ' . _("name") . ': "' . $oldPart->getName(true, true) . '"]', '8bit');
         $message->alterPart($partid, $newPart);
 
         /* We need to make sure we add "\r\n" after every line for
@@ -504,18 +481,16 @@ class IMP_Message {
      *   \\DELETED
      *   \\DRAFT
      *
-     * @access public
-     *
-     * @param string $flag              The IMAP flag(s) to set or clear.
-     * @param mixed &$indices           See above.
-     * @param optional boolean $action  True: set the flag(s) [DEFAULT];
-     *                                  false: clear the flag(s).
+     * @param string $flag     The IMAP flag(s) to set or clear.
+     * @param mixed &$indices  See above.
+     * @param boolean $action  If true, set the flag(s), otherwise clear the
+     *                         flag(s).
      *
      * @return boolean  True if successful, false if not.
      */
     function flag($flag, &$indices, $action = true)
     {
-        if (!($msgList = $this->_getMessageIndices($indices))) {
+        if (!($msgList = IMP::parseIndicesList($indices))) {
             return false;
         }
 
@@ -549,12 +524,10 @@ class IMP_Message {
      *   \\DELETED
      *   \\DRAFT
      *
-     * @access public
-     *
-     * @param string $flag              The IMAP flag(s) to set or clear.
-     * @param array $mboxes             The list of mailboxes to flag.
-     * @param optional boolean $action  True: set the flag(s) [DEFAULT];
-     *                                  false: clear the flag(s).
+     * @param string $flag     The IMAP flag(s) to set or clear.
+     * @param array $mboxes    The list of mailboxes to flag.
+     * @param boolean $action  If true, set the flag(s), otherwise, clear the
+     *                         flag(s).
      *
      * @return boolean  True if successful, false if not.
      */
@@ -583,8 +556,6 @@ class IMP_Message {
     /**
      * Expunges all deleted messages from the list of mailboxes.
      *
-     * @access public
-     *
      * @param array $mbox_list  The list of mailboxes to empty.
      */
     function expungeMailbox($mbox_list)
@@ -619,8 +590,6 @@ class IMP_Message {
     /**
      * Empties an entire mailbox.
      *
-     * @access public
-     *
      * @param array $mbox_list  The list of mailboxes to empty.
      */
     function emptyMailbox($mbox_list)
@@ -653,66 +622,4 @@ class IMP_Message {
         }
     }
 
-    /**
-     * Get message indices list.
-     *
-     * @access private
-     *
-     * @param mixed $indices  See above.
-     *
-     * @return mixed  Returns an array with the folder as key and an array
-     *                of message indices as the value (See #2 above).
-     *                Else, returns false.
-     */
-    function _getMessageIndices($indices)
-    {
-        global $imp;
-
-        $msgList = array();
-
-        if (is_a($indices, 'IMP_Mailbox')) {
-            $msgIdx = $indices->getIndex(true);
-            if (empty($msgIdx)) {
-                return false;
-            }
-            $parts = explode(IMP_IDX_SEP, $msgIdx);
-            $msgList[$parts[1]][] = $parts[0];
-            return $msgList;
-        }
-
-        if (!is_array($indices)) {
-            return false;
-        }
-        if (!count($indices)) {
-            return array();
-        }
-
-        reset($indices);
-        if (is_array(current($indices))) {
-            /* Build the list of indices/mailboxes to delete if input
-               is of format #1. */
-            foreach ($indices as $mbox => $msgs) {
-                foreach ($msgs as $msgIndex) {
-                    if (strpos($msgIndex, IMP_IDX_SEP) === false) {
-                        $msgList[$mbox][] = $msgIndex;
-                    } else {
-                        list($val, $key) = explode(IMP_IDX_SEP, $msgIndex);
-                        $msgList[$key][] = $val;
-                    }
-                }
-            }
-        } else {
-            /* We are dealing with format #2. Make sure we don't have
-               any duplicate keys. */
-            foreach ($indices as $key => $val) {
-                if (isset($msgList[$key])) {
-                    $msgList[$key] = array_unique(array_merge($val, $msgList[$key]));
-                } else {
-                    $msgList[$key] = is_array($val) ? $val : array($val);
-                }
-            }
-        }
-        return $msgList;
-    }
-
 }
