@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: imp/mailbox.php,v 2.617.4.3 2005/01/26 09:23:01 jan Exp $
+ * $Horde: imp/mailbox.php,v 2.617.4.21 2005/08/15 22:01:19 jan Exp $
  *
  * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
  * Copyright 1999-2005 Jon Parise <jon@horde.org>
@@ -147,6 +147,9 @@ $indices_mbox = array($imp['mailbox'] => $indices);
 /* Set the current time zone. */
 NLS::setTimeZone();
 
+/* Cache the charset. */
+$charset = NLS::getCharset();
+
 /* Initialize the user's identities. */
 $identity = &Identity::singleton(array('imp', 'imp'));
 
@@ -154,7 +157,7 @@ $identity = &Identity::singleton(array('imp', 'imp'));
 if (isset($from_message_page)) {
     $mailbox_url = Horde::applicationURL('mailbox.php');
 } else {
-    $mailbox_url = Horde::selfURL();
+    $mailbox_url = Horde::selfUrl();
 }
 
 /* Check for 'sortby' and 'sortdir' changes. */
@@ -169,12 +172,12 @@ if (($sortdir = Util::getFormData('sortdir')) !== null) {
 switch ($actionID) {
 case 'blacklist':
     require_once IMP_BASE . '/lib/Filter.php';
-    IMP_Filter::blacklistMessage($indices);
+    IMP_Filter::blacklistMessage($indices_mbox);
     break;
 
 case 'whitelist':
     require_once IMP_BASE . '/lib/Filter.php';
-    IMP_Filter::whitelistMessage($indices);
+    IMP_Filter::whitelistMessage($indices_mbox);
     break;
 
 case 'spam_report':
@@ -182,13 +185,17 @@ case 'notspam_report':
     $action = str_replace('_report', '', $actionID);
     require_once IMP_BASE . '/lib/Spam.php';
     $imp_spam = &new IMP_Spam();
-    $imp_spam->reportSpam($indices, $action);
+    $imp_spam->reportSpam($indices_mbox, $action);
 
     /* Delete spam after report. */
     if (($action == 'spam') && $prefs->getValue('delete_spam_after_report')) {
         if (_deleteMessages($indices_mbox)) {
-            $notification->push(sprintf(_("%d message(s) have been deleted."),
-                                        count($indices)), 'horde.message');
+            if (count($indices) == 1) {
+                $notification->push(_("1 message has been deleted."), 'horde.message');
+            } else {
+                $notification->push(sprintf(_("%d messages have been deleted."),
+                                            count($indices)), 'horde.message');
+            }
         }
     }
     break;
@@ -230,11 +237,11 @@ case 'copy_messages':
         } else {
             /* Otherwise, the target is a standard mailbox. */
             if (!empty($newMbox) && ($newMbox == 1)) {
-                $new_mailbox = IMP::addPreambleString(String::convertCharset($targetMbox, NLS::getCharset(), 'UTF7-IMAP'));
+                $new_mailbox = IMP::addPreambleString(String::convertCharset($targetMbox, $charset, 'UTF7-IMAP'));
 
                 require_once IMP_BASE . '/lib/Folder.php';
                 $imp_folder = &IMP_Folder::singleton();
-                if ($imp_folder->create($imp['stream'], $new_mailbox, $prefs->getValue('subscribe'))) {
+                if ($imp_folder->create($new_mailbox, $prefs->getValue('subscribe'))) {
                     $imp_message->copy($new_mailbox, $action, $indices_mbox);
                 }
             } else {
@@ -384,7 +391,13 @@ if ($preview_tooltip) {
     $strip_preview = $prefs->getValue('preview_strip_nl');
 }
 
-$unread = $imp_mailbox->unseenMessageCount();
+
+if ($search_mbox) {
+    $unread = 0;
+} else {
+    $unread = $imp_mailbox->unseenMessageCount();
+}
+
 $title = IMP::getLabel();
 $refresh_title = sprintf(_("_Refresh %s"), $title);
 $refresh_ak = Horde::getAccessKey($refresh_title);
@@ -440,7 +453,7 @@ if (empty($pageOb->end)) {
     exit;
 }
 
-/* If there is more than 15 messages , cache the actions output. */
+/* If there are more than 15 messages, cache the actions output. */
 if ($pageOb->msgcount != 0) {
     $navform = 1;
     require IMP_TEMPLATES . '/mailbox/navbar.inc';
@@ -544,7 +557,8 @@ foreach ($mailboxOverview as $msgIndex => $message) {
     /* Format the from header. */
     if (isset($h->from)) {
         $from_adr = IMP::bareAddress($h->from);
-        $from_ob = array_shift(imap_rfc822_parse_adrlist($h->from, ''));
+        $from_ob = imap_rfc822_parse_adrlist($h->from, '');
+        $from_ob = array_shift($from_ob);
         if (is_null($from_adr)) {
             $msg['from'] = _("Invalid Address");
             $showfromlink = false;
@@ -554,13 +568,14 @@ foreach ($mailboxOverview as $msgIndex => $message) {
                     $msg['from'] = _("Undisclosed Recipients");
                     $showfromlink = false;
                 } else {
-                    $tmp = array_shift(imap_rfc822_parse_adrlist($h->to, ''));
+                    $tmp = imap_rfc822_parse_adrlist($h->to, '');
+                    $tmp = array_shift($tmp);
                     if (isset($tmp->personal)) {
-                        $msg['from'] = MIME::decode($tmp->personal, NLS::getCharset());
+                        $msg['from'] = MIME::decode($tmp->personal, $charset);
                     } else {
                         $msg['from'] = IMP::bareAddress($h->to);
                     }
-                    $msg['fullfrom'] = MIME::decode($h->to, NLS::getCharset());
+                    $msg['fullfrom'] = MIME::decode($h->to, $charset);
                 }
             } else {
                 $msg['from'] = _("Undisclosed Recipients");
@@ -571,11 +586,11 @@ foreach ($mailboxOverview as $msgIndex => $message) {
             }
             $mailto_link = true;
         } elseif (isset($from_ob->personal)) {
-            $msg['from'] = trim(MIME::decode($from_ob->personal, NLS::getCharset()), '"');
+            $msg['from'] = trim(MIME::decode($from_ob->personal, $charset), '"');
             if (!trim($msg['from'], chr(160) . ' ')) {
                 $msg['from'] = $from_adr;
             }
-            $msg['fullfrom'] = MIME::decode($h->from, NLS::getCharset());
+            $msg['fullfrom'] = MIME::decode($h->from, $charset);
         } else {
             if (!isset($from_ob->host) ||
                 (strstr($from_ob->host, 'SYNTAX-ERROR') !== false)) {
@@ -583,7 +598,7 @@ foreach ($mailboxOverview as $msgIndex => $message) {
                 $showfromlink = false;
             } else {
                 $msg['from'] = $from_adr;
-                $msg['fullfrom'] = MIME::decode($h->from, NLS::getCharset());
+                $msg['fullfrom'] = MIME::decode($h->from, $charset);
             }
         }
     } else {
@@ -592,7 +607,10 @@ foreach ($mailboxOverview as $msgIndex => $message) {
     }
 
     if (!empty($h->subject)) {
-        $msg['subject'] = strtr(MIME::decode($h->subject, NLS::getCharset()), "\t", ' ');
+        $subject = MIME::decode($h->subject, $charset);
+        if (!empty($subject)) {
+            $msg['subject'] = strtr($subject, "\t", ' ');
+        }
     }
     $msg['fullsubject'] = $msg['subject'];
 
@@ -606,23 +624,27 @@ foreach ($mailboxOverview as $msgIndex => $message) {
     $msg['subject'] = IMP::filterText($msg['subject']);
     $msg['fullsubject'] = IMP::filterText($msg['fullsubject']);
 
-    if (isset($message['structure'])) {
-        if (!empty($message['structure']->subtype)) {
-            if (String::lower($message['structure']->subtype) == 'signed' ||
-                String::lower($message['structure']->subtype) == 'encrypted') {
-                $attachment = 'signed';
-            } elseif (String::lower($message['structure']->subtype) == 'alternative') {
-                /* Treat this as no attachments. */
-            } elseif (isset($message['structure']->parts) &&
-                      is_array($message['structure']->parts) &&
-                      count($message['structure']->parts) > 0) {
-                $attachment = 'attachment';
-            } elseif ((!empty($message['structure']->type) &&
-                       $message['structure']->type != TYPETEXT) ||
-                      !empty($message['structure']->subtype) &&
-                      String::lower($message['structure']->subtype) != 'plain') {
-                $attachment = 'attachment';
-            }
+    if (isset($message['structure']) &&
+        ($message['structure']->getPrimaryType() == 'multipart')) {
+        switch ($message['structure']->getSubType()) {
+        case 'signed':
+            $attachment = 'signed';
+            $attachment_alt = _("Message is signed");
+            break;
+
+        case 'encrypted':
+            $attachment = 'encrypted';
+            $attachment_alt = _("Message is encrypted");
+            break;
+
+        case 'alternative':
+            /* Treat this as no attachments. */
+            break;
+
+        default:
+            $attachment = 'attachment';
+            $attachment_alt = _("Message has attachments");
+            break;
         }
     }
 
@@ -682,7 +704,9 @@ foreach ($mailboxOverview as $msgIndex => $message) {
             $msg['bg'] = 'important';
         } elseif ($xprio == 'low') {
             $msg['status'] .= Horde::img('mail_priority_low.png', _("Low Priority"));
-            $style[] = 'i';
+            if (isset($style[0]) && $style[0] == 'b') {
+                $style = array();
+            }
         }
         if ($h->flagged) {
             $flagbits |= IMP_FLAGGED;
@@ -706,41 +730,41 @@ foreach ($mailboxOverview as $msgIndex => $message) {
     }
 
 
-    /* Set the message number. */
-    $msg['number'] = $h->msgno;
-
     /* Show message preview? */
     if ($show_preview) {
         $msg['preview'] = $message['preview'];
-        if (!$preview_tooltip) {
-            if (!$strip_preview) {
-                $msg['preview'] = str_replace(' ', '&nbsp;', $msg['preview']);
-            }
-            $msg['preview'] = nl2br($msg['preview']);
-        }
     }
 
+    /* Set the message number. */
+    $msg['number'] = _stylize($h->msgno, $style);
+
     /* Format the Date: Header. */
     $msg['date'] = _stylize($msg['date'], $style);
 
+    /* Format message size. */
+    $msg['size'] = _stylize($msg['size'], $style);
+
     /* Format the From: Header. */
     $msg['from'] = _stylize(htmlspecialchars($msg['from']), $style);
-    if ($showfromlink) {
-        switch ($fromlinkstyle) {
-        case 0:
+    switch ($fromlinkstyle) {
+    case 0:
+        if ($showfromlink) {
             $extra = array('actionID' => 'mailto', 'mailbox' => $imp['mailbox'], 'index' => $h->uid, 'mailto' => $mailto_link);
             if ($search_mbox) {
                 $extra['thismailbox'] = $mailboxOverview[$msgIndex]['mbox'];
             }
             $msg['from'] = Horde::link(IMP::composeLink(array(), $extra), sprintf(_("Compose Message (%s)"), $msg['fullfrom'])) . $msg['from'] . '</a>';
-            break;
-
-        case 1:
-            $from_uri = IMP_Search::generateSearchUrl('message.php', $msgMbox);
-            $from_uri = Util::addParameter($from_uri, 'index', $h->uid);
-            $msg['from'] = Horde::link($from_uri, $msg['fullfrom']) . $msg['from'] . '</a>';
-            break;
         }
+        break;
+
+    case 1:
+        if (!isset($msg['fullfrom'])) {
+            $msg['fullfrom'] = $msg['from'];
+        }
+        $from_uri = IMP_Search::generateSearchUrl('message.php', $msgMbox);
+        $from_uri = Util::addParameter($from_uri, 'index', $h->uid);
+        $msg['from'] = Horde::link($from_uri, $msg['fullfrom']) . $msg['from'] . '</a>';
+        break;
     }
 
     /* Format the Subject: Header. */
